@@ -154,7 +154,10 @@ SEXP C_write_data_frame_list(SEXP df_list, SEXP file, SEXP col_names, SEXP forma
         case COL_POSIXCT: {
           double val = REAL(col)[i];
           if(R_FINITE(val))
-            assert_lxw(worksheet_write_number(sheet, cursor, j, 25569 + val / (24*60*60) , NULL));
+            val = 25568.0 + val / (24*60*60);
+            if(val >= 60.0)
+              val = val + 1.0;
+            assert_lxw(worksheet_write_number(sheet, cursor, j, val , NULL));
         }; continue;
         case COL_STRING:{
           SEXP val = STRING_ELT(col, i);
@@ -205,7 +208,7 @@ SEXP C_write_data_frame_list(SEXP df_list, SEXP file, SEXP col_names, SEXP forma
   return file;
 }
 
-SEXP C_lxw_version(){
+SEXP C_lxw_version(void){
   return Rf_mkString(LXW_VERSION);
 }
 
